-- Created by hey @ Widow.
-- Licensed under the WTFPL.  See http://www.wtfpl.net/txt/copying/ for details.

-- Type "/iqc toggle combat" to toggle whether or not turn-ins will be completed during combat.
-- Type "/iqc toggle popup" to toggle whether or not popup quests will be auto-accepted.

local InstantQuestComplete = {}

local disable_in_combat = true
local disable_mention_accept = false
local is_in_combat = false
local queued_turnins = {}


function string.split(str, pattern)
  local tokens = {}

  for token in string.gmatch(str, pattern or "%S+") do
    table.insert(tokens, token)
  end

  return tokens
end


function InstantQuestComplete:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self

  return o
end

function InstantQuestComplete:Init()
  Apollo.RegisterAddon(self)

  Apollo.RegisterSlashCommand("iqc", "OnSlashCommand", self)

  Apollo.RegisterEventHandler("QuestStateChanged", "DoTurnIn", self)
  Apollo.RegisterEventHandler("QuestObjectiveUpdated", "DoTurnIn", self)


  Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)
  Apollo.RegisterTimerHandler("IQCTurnIn", "OnTurnInTimerTick", self)
  Apollo.CreateTimer("IQCTurnIn", 1, true)
end

function InstantQuestComplete:OnSave(level)
  if level ~= GameLib.CodeEnumAddonSaveLevel.Character then
    return nil
  end

  return { ["disable_in_combat"] = disable_in_combat, ["disable_mention_accept"] = disable_mention_accept }
end

function InstantQuestComplete:OnRestore(level, save)
  if level ~= GameLib.CodeEnumAddonSaveLevel.Character then
    return nil
  end

  disable_in_combat = save["disable_in_combat"]
  disable_mention_accept = save["disable_mention_accept"]
end

function InstantQuestComplete:OnSlashCommand(_, argstr)
  local args = string.split(argstr)

  if #args == 0 then
    self:PrintHelp()
    self:PrintToggleHelp()
    return
  end

  local cmd = args[1]
  if cmd == "help" then
    self:PrintHelp()
    self:PrintToggleHelp()
  elseif cmd == "toggle" then
    if #args ~= 2 then
      Print("InstantQuestComplete will" .. (disable_in_combat and " not" or "") .. " complete quests while in combat.")
      Print("InstantQuestComplete will" .. (disable_mention_accept and " not" or "") .. " auto-accept quest popups.")

      return
    end

    if args[2] == "combat" then
      disable_in_combat = not disable_in_combat
      Print("InstantQuestComplete will" .. (disable_in_combat and " not" or "") .. " complete quests while in combat.")
    elseif args[2] == "popup" then
      disable_mention_accept = not disable_mention_accept
      Print("InstantQuestComplete will" .. (disable_mention_accept and " not" or "") .. " auto-accept quest popups.")
    else
      self:PrintToggleHelp()
    end
  end
end

function InstantQuestComplete:PrintHelp()
  Print("InstantQuestComplete - Commands")
  Print("  \"/iqc help\" - Show this message.")
end

function InstantQuestComplete:PrintToggleHelp()
  Print("  \"/iqc toggle [combat|popup]\"")
  Print("    \"combat\": Enable/disable turn-ins while in combat.")
  Print("    \"popup\": Enable/disable auto-accepting quest popups.")
  Print("    no argument: Show current settings.")
end

function InstantQuestComplete:DoTurnIn(quest, junk)
  local state = quest:GetState()

  if state == Quest.QuestState_Achieved or (not disable_mention_accept and state == Quest.QuestState_Mentioned) then
    local contact = quest:GetContactInfo()

    if contact and contact.strName and string.len(contact.strName) > 0 then
      if disable_in_combat and is_in_combat then
        queued_turnins[quest:GetId()] = quest
      else
        CommunicatorLib.CallContact(quest)
      end
    end
  end
end

function InstantQuestComplete:OnEnteredCombat(unit, in_combat)
  if unit and unit:IsThePlayer() then
    is_in_combat = in_combat
  end
end

function InstantQuestComplete:OnTurnInTimerTick()
  if is_in_combat then
    return
  end

  for id, quest in pairs(queued_turnins) do
    if quest then
      CommunicatorLib.CallContact(quest)

       -- probably don't have to worry about this as we are just reassigning to nil, and not removing the key itself
      queued_turnins[id] = nil
    end
  end
end

local InstantQuestCompleteInst = InstantQuestComplete:new()
InstantQuestCompleteInst:Init()
